/* delete.c - CKS Key Deletion Application main source file
 * Copyright (C) 2001-2005 CryptNET, V. Alex Brennen (VAB)
 *
 * This file is part of the CryptNET OpenPGP Public Key Server (cks).
 *
 * cks is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * cks is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "delete.h"


int main(void)
{
	struct cks_ctx *ctx = NULL;
	struct name_value_pair_dllst *form = NULL;

	PGconn *conn = NULL;
	
	char *fingerprint = NULL;
	char *reject_key = NULL;

	char *method = NULL;
	char *content = NULL;
	unsigned long content_length = 0;

	char stmt[300];
	int result = 0;
	int rslt = 0;
	int reject = 0;


	method = getenv("REQUEST_METHOD");
	if(method == NULL)
	{
		do_error_page(_("Request Method was Null.\n<P><P>Exiting..."));

		return -1;
	}
	else if(strcmp(method,"GET") == 0)
	{
		do_error_page(_("This program should be called with the POST method."));

		return -1;
	}
	else if(strcmp(method,"POST") == 0)
	{
		content_length = atoi(getenv("CONTENT_LENGTH"));
		if(content_length > 300)
		{
			do_error_page(_("Content Length expectation exceeded\n"));

			return -1;
		}
		content = (char *)malloc(content_length+1);
		if(content == NULL)
		{
			do_error_page(_("Server was unable to malloc memory.  Server out of memory."));

			return -1;
		}
		fread(content,1,content_length,stdin);
	}
	else
	{
		do_error_page(_("Unknown Method."));

		return -1;
	}

	if(hex_to_ascii(content) == -1)
	{
		fprintf(stderr,"hex_to_ascii failure\n");
		if(content != NULL)
			free(content);

		return -1;
	}
	form = parse_name_value_pairs(content);
	if(form == NULL)
	{
		do_error_page(_("Failed to process form\n"));
		if(content != NULL)
			free(content);

		return -1;
	}
	fingerprint = get_value(form,"fp");
	reject_key = get_value(form,"reject");
	
	if( (fingerprint == NULL) || (reject_key == NULL) )
	{
		print_admin_header(_("Error:  Invalid Fingerprint"));
		printf(_("The Fingerprint that you provided is invalid.\n"));
		print_admin_footer();
		if(form != NULL)
		{
			free_name_value_pair_dllst(form);
		}

		return -1;
	}

	if( (strlen(fingerprint) != 32) && (strlen(fingerprint) != 40) &&
	    (strlen(fingerprint) != 48) && (strlen(fingerprint) != 50)  )
	{
		print_admin_header(_("Error:  Invalid Fingerprint"));
		printf(_("The Fingerprint that you provided (%s) is invalid.\n"),fingerprint);
		print_admin_footer();
		if(form != NULL)
		{
			free_name_value_pair_dllst(form);
		}

		return -1;
	}

	ctx = (struct cks_ctx *)malloc(sizeof(struct cks_ctx));
	if(ctx == NULL)
	{
		fprintf(stderr,_("cks_delete:  Fatal Error:  Malloc call failed: Out of Memorry.\n"));
		if(form != NULL)
		{
			free_name_value_pair_dllst(form);
		}

		return -1;
	}
	ctx->config = (struct cks_config *)malloc(sizeof(struct cks_config));
	if(ctx->config == NULL)
	{
		fprintf(stderr,"delete: malloc call failed.\n");
		if(form != NULL)
		{
			free_name_value_pair_dllst(form);
		}
		free_context(ctx);
		if(content != NULL)
			free(content);

		return -1;
	}
	rslt = init_config(&ctx,0);
	if(rslt == -1)
	{
		fprintf(stderr,_("delete:  Non-Fatal Error: Failed to read config.\n"));
		fprintf(stderr,_("delete:  Using default configuration information.\n"));
	}

	conn = db_connect(ctx->config);
	if(conn == NULL)
	{
		fprintf(stderr,"Failed to connect to the Database.\n");
		if(form != NULL)
		{
			free_name_value_pair_dllst(form);
		}
		print_admin_footer();
		if(content != NULL)
			free(content);
		free_context(ctx);

		return -1;
	}

	remove_spaces(&fingerprint);

	if(!key_exists(conn,fingerprint))
	{
		print_admin_header(_("Error:  Key Not Found In DB"));
		printf(_("The key with the fingerprint '%s' was not found in the database\n"),fingerprint);
		printf(_("and therefor could not be deleted.\n"));
		print_admin_footer();
		if(form != NULL)
		{
			free_name_value_pair_dllst(form);
		}
		if(content != NULL)
			free(content);
		free_context(ctx);

		return -1;
	}

	if(key_rejected(conn,fingerprint))
	{
		print_admin_header(_("Error:  This key has already been deleted.\n"));
		printf(_("The key with the fingerprint '%s' has already been deleted and added to the\n"),fingerprint);
		printf(_("list of rejected keys.\n"));
		print_admin_footer();
		if(form != NULL)
		{
			free_name_value_pair_dllst(form);
		}
		if(content != NULL)
			free(content);
		free_context(ctx);

		return -1;
	}

	stmt[0] = '\0';

	if(db_begin_transaction(conn) == -1)
	{
		fprintf(stderr,"Failed to start db transaction.\n");
		db_disconnect(conn);
		if(form != NULL)
		{
			free_name_value_pair_dllst(form);
		}
		if(content != NULL)
			free(content);
		free_context(ctx);

		return -1;
	}

	rslt = delete_key_from_db(conn,fingerprint,0);
	if(rslt == -1)
	{
		print_admin_header(_("Error."));
		printf(_("Failed to delete key.\n"));
		print_admin_footer();
		if(form != NULL)
		{
			free_name_value_pair_dllst(form);
		}
		if(content != NULL)
			free(content);
		free_context(ctx);

		return -1;
	}

	if(memcmp(reject_key,"YES",3) == 0)
	{
		reject = 1;
	}

	if(reject == 1)
	{
		memset(stmt,0x00,200);
		snprintf(stmt,200,"insert into cks_rejected_keys values('%s')", fingerprint);
		result = db_stmt(conn,stmt,ctx->config);
		if(result == -1)
		{
			do_error_page(_("Error inserting fp into cks_rejected_keys."));
			if(form != NULL)
			{
				free_name_value_pair_dllst(form);
			}
			if(content != NULL)
				free(content);
			free_context(ctx);

			return -1;
                }

		if(db_commit_transaction(conn) == -1)
		{
			do_error_page("Failed To commit transaction.\n");
			if(form != NULL)
			{
				free_name_value_pair_dllst(form);
			}
			if(content != NULL)
				free(content);
			free_context(ctx);

			return -1;
		}

		print_admin_header(_("Key Deleted"));
		printf(_("The key %s has been deleted from the database and inserted into the\n"), fingerprint);
		printf(_("cks_rejected_keys table.  Attempts to add this key back into the database\n"));
		printf(_("will fail.  If you would like to add this key again, you must delete that\n"));
		printf(_("record from the cks_rejected_keys table.\n"));
		print_admin_footer();
	}
	else
	{
		if(db_commit_transaction(conn) == -1)
		{
			do_error_page("Failed To commit transaction.\n");
			if(form != NULL)
			{
				free_name_value_pair_dllst(form);
			}
			if(content != NULL)
				free(content);
			free_context(ctx);

			return -1;
		}

		print_admin_header(_("Key Deleted"));
		printf(_("The key %s has been deleted from the database.\n"), fingerprint);
		print_admin_footer();
	}

	if(form != NULL)
	{
		free_name_value_pair_dllst(form);
	}
	if(content != NULL)
	{
		free(content);
	}
	free_context(ctx);

	return 0;
}
